<?php

/* Registro00GRRF.
 * @package Logic
 * @author  francismararaujo
 */
class Registro00GRRF extends RegistroGRRF{

    const REGISTRO_00 = '00';
    const GRRF = '2';
    // Implementacao futura.
    const DERF = '3';
    const MODALIDADE_BRANCO = 'BRANCO';

    private $_pessoaResp;
    private $_estruturaResp;
    private $_nrRecolhimento;
    private $_modalidade;
    private $_competencia13;
    private $_nrInscricao;
    private $_tpInscricao;
    private $_dtRecolhGRRF;
    private $_idRecolhimentoFGTS;
    private $_dtFGTS;
    private $_idRecolhimentoGPS;
    private $_dtGPS;

    /* Preparacao para a criacao do Registro 00.
     * @param GpepessoaEntity $pessoa      Instancia de GpepessoaEntity.
     * @param string          $tpInscricao Tipo de inscricao da empresa responsavel pela geracao do arquivo.
     * @param string          $nrInscricao Numero de inscricao da empresa responsavel pela geracao do arquivo.
     * @return void
     * @author francismararaujo
     */

    public function geraRegistro00(){
        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_modalidade = $this->getModalidade();
        $this->_competencia13 =  $this->getCompetencia13();
        $this->_geraRegistro(self::REGISTRO_00);
    }

    /* Cria o Registro 00.
     * @param string $tpRegistro  Tipo de registro que sera criado.
     * @param string $tpInscricao Tipo de inscricao da empresa.
     * @param string $nrInscricao Numero de inscricao da empresa.
     * @return void
     * @author francismararaujo
     */

    private function _geraRegistro($tpRegistro){

        //bloco 1 | 001-002 | tipo do registro.
        $this->atribuiDado($tpRegistro, 2);

        //bloco 2 | 003-053 | brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 51, parent::CARACTER_BRANCO);

        //bloco 3 | 054 | tipo de remessa.
        $this->atribuiDado(self::GRRF, 1);

        //bloco 4 | 055 | tipo de inscricao.
        $this->atribuiDado($this->_tpInscricao, 1);

        //bloco 5 | 056-069 | numero inscricao do responsavel.
        $this->atribuiDado($this->_nrInscricao, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 6 | 070-099 | nome do responsavel.
        //@todo Verificar se pode vir como parametro no formulario.
        $nomeResponsavel = $this->_estruturaResp->historico->getNmrazsocestrut();
        $this->atribuiDado($nomeResponsavel, 30, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 7 | 100-119 | nome pessoa contato.
        //@todo Verificar se pode vir como parametro no formulario.
        $nomePessoa = $this->_pessoaResp->historico->getNmpessoa();
        $this->atribuiDado($nomePessoa, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 8 | 120-169 | endereco do responsavel.
        //@todo Verificar se pode vir como parametro no formulario.
        $endereco = $this->_estruturaResp->endereco->getDsendereco();
        $this->atribuiDado($endereco, 50, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 9 | 170-189 | bairro do responsavel.
        //@todo Verificar se pode vir como parametro no formulario.
        $bairro = $this->_estruturaResp->endereco->bairro;
        $this->atribuiDado($bairro, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 10 | 190-197 | cep.
        //@todo Diferentes de 20000000, 30000000, 70000000 ou 80000000.
        //@todo Verificar se pode vir como parametro no formulario.
        $cep = $this->_estruturaResp->endereco->getNrcependereco();
        $this->atribuiDado($cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 11 | 198-217 | cidade.
        //@todo Verificar se pode vir como parametro no formulario.
        $cidade = $this->_estruturaResp->endereco->municipio;
        $this->atribuiDado($cidade, 20, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 12 | 218-219 | unidade da federacao (estado).
        //@todo Verificar se pode vir como parametro no formulario.
        $estado = $this->_estruturaResp->endereco->getSgestado();
        $this->atribuiDado($estado, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 13 | 220-231 | telefone contato.
        //@todo Verificar se pode vir como parametro no formulario.
        $telefone = $this->_estruturaResp->endereco->ddd.$this->_estruturaResp->endereco->telefone;
        $this->atribuiDado($telefone, 12, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        //bloco 14 | 232-291 | endereco eletronico (e-mail).
        //@todo Verificar se pode vir como parametro no formulario.
        $enderecoEletronico = $this->_estruturaResp->endereco->email;
        $this->atribuiDadoEspecial($enderecoEletronico, 60, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        //bloco 15 | 292-299 | data recolhimento GRRF
        $this->_atribuiDataRecolhimentoGRRF();

        //bloco 16 | 300-359 | preencher com brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 60, parent::CARACTER_BRANCO);

        //bloco 17 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);

    }

    public function setNrInscricao($nrInscricao){
        $this->_nrInscricao = $nrInscricao;
    }

    public function setTpInscricao($tpInscricao){
        $this->_tpInscricao = $tpInscricao;
    }

    public function setPessoaResp($pessoaResp){
        $this->_pessoaResp = $pessoaResp;
    }

    public function setEstruturaResp($estruturaResp){
        $this->_estruturaResp = $estruturaResp;
    }

    public function setIdRecolhimentoFGTS($idRecolhimentoFGTS) {
        $this->_idRecolhimentoFGTS = $idRecolhimentoFGTS;
    }

    public function setDtFGTS($dtFGTS) {
        $this->_dtFGTS = $dtFGTS;
    }

    public function setDtRecolhGRRF($dtRecolhGRRF) {
        $this->_dtRecolhGRRF = $dtRecolhGRRF;
    }

    public function setIdRecolhimentoGPS($idRecolhimentoGPS) {
        $this->_idRecolhimentoGPS = $idRecolhimentoGPS;
    }

    public function setDtGPS($dtGPS) {
        $this->_dtGPS = $dtGPS;
    }

    /* Enter description here ...
     * @return void
     * @author francismararaujo - 26/02/2013
     */

    private function _atribuiDataRecolhimentoGRRF(){

        /* Campo Obrigatório
         * Data válida
         * Informar a data em que se pretende efetuar o pagamento das guiasrescisória.
         * Deve estar dentro do período de vigência do edital (Tabela de Índices do FGTS)
carregado.
         */

        $dataGRRF = parent::CARACTER_BRANCO;
        $dataGRRF = new TkDateTime($this->_dtRecolhGRRF);
        $dataGRRF = $dataGRRF->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
        $this->atribuiDado($dataGRRF, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }

    /* Atribui Recolhimento FGTS.
     * @return void
     * @author alessandrobartels
     *         alessandrobartels - 17/04/2012 - validacao
     */

    private function _atribuiRecolhimentoFGTS(){

        if (($this->_nrRecolhimento !== '211') && ($this->_competencia13 === false) ) {
            $idRecolhimentoFGTS = $this->_idRecolhimentoFGTS;
        } else {
            $idRecolhimentoFGTS = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($idRecolhimentoFGTS, 1);
    }

    /* Enter description here ...
     * @return void
     * @author alessandrobartels
     */

    private function _atribuiModalidade(){
        // @todo verificar layout e arquivo ARTE para validacoes.
        if ($this->_modalidade === self::MODALIDADE_BRANCO){
            $this->_modalidade = self::CARACTER_BRANCO;
        }

        $this->atribuiDado($this->_modalidade, 1);
    }

    /* Enter description here ...
     * @return void
     * @author alessandrobartels - 17/04/2012
     */

    private function _atribuiDataRecolhimentoFGTS(){

        $dataFGTS = parent::CARACTER_BRANCO;
        if ($this->_idRecolhimentoFGTS !== 1) {
            if (empty($this->_dtFGTS) !== true){
                $dataFGTS = new TkDateTime($this->_dtFGTS);
                $dataFGTS = $dataFGTS->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
            } 
        }
        $this->atribuiDado($dataFGTS, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }

    
    /* Enter description here ...
     * @return void
     * @author alessandrobartels
     *         alessandrobartels - 17/04/2012 - validacao
     */

    private function _atribuiRecolhimentoGPS(){
        // @todo verificar layout e arquivo ARTE para validacoes.
        // (1 = no prazo | 2 = em atraso | 3 = nao gera GPS).
            $this->atribuiDado($this->_idRecolhimentoGPS, 1);
    }

    /* Enter description here ...
     * @return void
     * @author alessandrobartels - 17/04/2012
     */

    private function _atribuiDataRecolhimentoGPS(){

        $dataGPS = parent::CARACTER_BRANCO;
        if ($this->_idRecolhimentoGPS !== 1) {
            if (empty($this->_dtGPS) !== true){
                $dataGPS = new TkDateTime($this->_dtGPS);
                $dataGPS = $dataGPS->toString(TkDateTime::DAY.TkDateTime::MONTH.TkDateTime::YEAR);
            }
        }
        $this->atribuiDado($dataGPS, 8, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }

    /* Enter description here ...
     * @param string $tipoInscricaoFornecedor Tipo de inscricao do fornecedor.
     * @return void
     * @author alessandrobartels
     */

    private function _atribuiTipoInscricaoFornecedor($tipoInscricaoFornecedor){
        // @todo verificar layout e arquivo ARTE para validacoes.
        $this->atribuiDado($tipoInscricaoFornecedor, 1); // Criar constante.
    }

}