import {
  IEventParam, Metadata, Loader, IDictionary, Config,
} from '@zeedhi/core';
import {
  Select, Form, Radio, Button, SelectMultiple, CheckboxMultiple, SelectTree,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pla01303EmissaoReceitasController {
  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');
  
  public appController = Loader.getInstance('AppController');
    
  public reportController = Loader.getInstance('ReportController');
  
  public setUnidade = false;

  public project: string = 'PLA';
  
  public central = this.appController.getCdCentral();
  
  public inicContainer() {
    // const project = localStorage.getItem('PRODUCT_ID');
    
    /* const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE; */
    const idTipoRece = Metadata.getInstance<Radio>('IDTIPORECE');
    
    if (this.utilitiesPLA.isMerenda()) this.project = 'PAE';
    else {
      this.project = 'PLA';
      const dataRece = idTipoRece?.datasource?.data;
      if (dataRece?.length === 4) {
        dataRece?.splice(3, 1);
      }
    }
  }
  
  private impStatusProduto = {
    E: 'E',
    D: 'D',
    P: 'P',
  };
  
  public setStatusProduto({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('E')) this.impStatusProduto.E = 'E';
      else this.impStatusProduto.E = 'X';

      if (items.includes('D')) this.impStatusProduto.D = 'D';
      else this.impStatusProduto.D = 'X';
      
      if (items.includes('P')) this.impStatusProduto.P = 'P';
      else this.impStatusProduto.P = 'X';
    }
  }

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setValues() {
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE;
    const origem = Metadata.getInstance<Radio>('IDTIPORECE');
    origem.isVisible = true;
    origem.disabled = false;

    const grupfili = Metadata.getInstance<Select>('GRUPFILI');
    grupfili.isVisible = false;
    
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    unidade.isVisible = true;
    unidade2.isVisible = false;

    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico1.isVisible = true;
    servico2.isVisible = false;
    
    const tpcard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpcard2 = Metadata.getInstance<Select>('CDTIPOCARD2');
    
    if (this.project === 'PLA') {
      tpcard1.isVisible = false;
      tpcard2.isVisible = false;
    } else {
      tpcard1.isVisible = true;
      tpcard2.isVisible = false;
    }
  
    const ordena = Metadata.getInstance<Radio>('ORDENA');
    const ordenaAlf = Metadata.getInstance<Select>('NIVEL');
    ordena.isVisible = false;
    ordenaAlf.isVisible = false;
    
    const pratoini = Metadata.getInstance<SelectTree>('PRATO_INICIAL');
    const pratofim = Metadata.getInstance<SelectTree>('PRATO_FINAL');
    pratoini.isVisible = true;
    pratofim.isVisible = true;
    pratoini.addValidation('required', { message: 'pme::campoObrigatorio' });
    pratofim.addValidation('required', { message: 'pme::campoObrigatorio' });
    
    const prodini = Metadata.getInstance<SelectTree>('PRODINI');
    const prodfim = Metadata.getInstance<SelectTree>('PRODFIN');
    prodini.isVisible = false;
    prodfim.isVisible = false;
    prodini.removeValidation('required');
    prodfim.removeValidation('required');
    
    const tpReq = Metadata.getInstance<Select>('IDTPREQPRODFILI');
    tpReq.isVisible = false;
    tpReq.removeValidation('required');
      
    const nutri = Metadata.getInstance<Select>('CDNUTRIENTE');
    nutri.isVisible = false;
    
    const dtini = Metadata.getInstance<Select>('DTINI');
    const dtfin = Metadata.getInstance<Select>('DTFIN');
    dtfin.isVisible = false;
    dtini.isVisible = false;
    dtini.removeValidation('required');
    dtfin.removeValidation('required');
    
    // inicializa receitas
    grupfili.disabled = true;
    unidade.disabled = true;
    unidade2.disabled = true;
    servico1.disabled = true;
    servico2.disabled = true;
    tpcard1.disabled = true;
    tpcard2.disabled = true;
    
    if (report === 'PLA01300') {
      origem.disabled = true;
      unidade2.removeValidation('required');
      
      ordena.isVisible = true;
      ordenaAlf.isVisible = true;
      ordenaAlf.disabled = true;
    } else if (report === 'PLA01305') {
      origem.disabled = true;
      unidade2.removeValidation('required');
      ordenaAlf.removeValidation('required');
      dtfin.isVisible = true;
      dtini.isVisible = true;
      dtini.addValidation('required', { message: 'pme::campoObrigatorio' });
      dtfin.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla01306') {
      origem.disabled = true;
      unidade.isVisible = false;
      unidade2.isVisible = true;
      unidade2.disabled = false;
      unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla05500') {
      const unidadePrato = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM;
      
      const values = {
        CDFILIAL: unidadePrato.CDFILIAL,
        NMFILIAL: unidadePrato.NMFILIAL,
      };
      
      unidade2.datasource.data = [values];
      unidade2.setValue(unidadePrato.CDFILIAL);
      
      grupfili.isVisible = true;
      unidade2.removeValidation('required');
      
      prodini.isVisible = true;
      prodfim.isVisible = true;
      prodini.addValidation('required', { message: 'pme::campoObrigatorio' });
      prodfim.addValidation('required', { message: 'pme::campoObrigatorio' });
      
      pratoini.isVisible = false;
      pratofim.isVisible = false;
      pratoini.removeValidation('required');
      pratofim.removeValidation('required');
      this.changeOrigem(report);
    } else if (report === 'pla05000') {
      unidade.isVisible = false;
      unidade2.isVisible = true;
      unidade2.removeValidation('required');
      servico1.isVisible = false;
      servico2.isVisible = true;

      if (this.project === 'PLA') {
        tpcard1.isVisible = false;
        tpcard2.isVisible = false;
      } else {
        tpcard1.isVisible = false;
        tpcard2.isVisible = true;
      }
      
      nutri.isVisible = true;
      
      ordena.isVisible = true;
      ordenaAlf.isVisible = true;
      ordenaAlf.disabled = true;
      
      this.changeOrigem(report);
    } else if (report === 'pla05400') {
      origem.isVisible = true;
      origem.disabled = true;
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      
      grupfili.isVisible = true;
      grupfili.disabled = false;
      unidade.disabled = false;
      
      prodini.isVisible = true;
      prodfim.isVisible = true;
      prodini.addValidation('required', { message: 'pme::campoObrigatorio' });
      prodfim.addValidation('required', { message: 'pme::campoObrigatorio' });
      ordenaAlf.removeValidation('required');
      tpReq.isVisible = true;
      // tpReq.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla013B') {
      unidade.isVisible = false;
      unidade2.isVisible = true;
      // unidade2.removeValidation('required');

      servico1.isVisible = false;
      servico2.isVisible = true;
      
      if (this.project === 'PLA') {
        tpcard1.isVisible = false;
        tpcard2.isVisible = false;
      } else {
        tpcard1.isVisible = false;
        tpcard2.isVisible = true;
      }
      
      ordenaAlf.removeValidation('required');
      nutri.isVisible = true;
      
      this.changeOrigem(report);
    }
    
    this.clearFields();
  }
  
  public changeOrigem(report: string) {
    const origem = Metadata.getInstance<Radio>('IDTIPORECE');
    
    const grupfili = Metadata.getInstance<Select>('GRUPFILI');
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    unidade2.removeValidation('required');
    
    const servico1 = Metadata.getInstance<SelectMultiple>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico2.removeValidation('required');
    
    const tpcard1 = Metadata.getInstance<SelectMultiple>('CDTIPOCARD1');
    const tpcard2 = Metadata.getInstance<Select>('CDTIPOCARD2');
    tpcard2.removeValidation('required');

    if (origem.value === 'P') {
      grupfili.disabled = true;
      unidade.disabled = true;
      unidade2.disabled = true;
      servico1.disabled = true;
      servico2.disabled = true;
      
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      tpcard2.removeValidation('required');
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
      
      if (report === 'pla05500') {
        unidade.isVisible = false;
        unidade2.isVisible = true;
        unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
        unidade2.disabled = false;
      } else {
        unidade.isVisible = true;
        unidade2.isVisible = false;
        unidade2.disabled = true;
        unidade2.removeValidation('required');
      }
    } else if (origem.value === 'F') {
      grupfili.disabled = false;
      unidade.disabled = false;
      unidade2.disabled = false;
      servico1.disabled = true;
      servico2.disabled = true;
      tpcard2.removeValidation('required');

      if (report === 'pla05000' || report === 'pla013B') {
        unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
        servico2.removeValidation('required');
      } else {
        unidade2.removeValidation('required');
        servico2.removeValidation('required');
      }
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
    } else if (origem.value === 'S') {
      grupfili.disabled = false;
      unidade.disabled = false;
      unidade2.disabled = false;
      servico1.disabled = false;
      servico2.disabled = false;
      tpcard2.removeValidation('required');
      
      if (report === 'pla05000' || report === 'pla013B') {
        unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
        servico2.addValidation('required', { message: 'pme::campoObrigatorio' });
      } else {
        unidade2.removeValidation('required');
        servico2.removeValidation('required');
      }
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
    } else if (origem.value === 'T') {
      grupfili.disabled = true;
      unidade.disabled = true;
      unidade2.disabled = true;
      servico1.disabled = true;
      servico2.disabled = true;
      
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      
      tpcard1.disabled = false;
      tpcard2.disabled = false;
      
      if (report === 'pla05000' || report === 'pla013B') {
        tpcard2.addValidation('required', { message: 'pme::campoObrigatorio' });
      } else {
        tpcard2.removeValidation('required');
      }
    }
  }
  
  public clearUnidade() {
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    
    unidade.setValue('');
    unidade2.setValue('');
    
    this.clearServico();
  }
  
  public clearServico() {
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico1.setValue('');
    servico2.setValue('');
  }
  
  public clearFields() {
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    const tpCard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpCard2 = Metadata.getInstance<Select>('CDTIPOCARD2');

    this.utilities.clearPeriodoParams();

    servico1.setValue('');
    servico2.setValue('');
    unidade.setValue('');
    tpCard1.setValue('');
    tpCard2.setValue('');
  }
  
  public setFilterTpCardapio() {
    const cardapio1 = Metadata.getInstance<SelectMultiple>('CDTIPOCARD1');
    const cardapio2 = Metadata.getInstance<SelectMultiple>('CDTIPOCARD2');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE;
    
    if (report === 'pla05500') {
      cardapio1.datasource.filter = filter;
      cardapio1.datasource.get();
    } else {
      cardapio2.datasource.filter = filter;
      cardapio2.datasource.get();
    }
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const filter = [
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    component.datasource.loading = true;
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE;
    let unidade;
    let unidadeValue;
    
    if (report === 'pla05000' || report === 'pla013B') {
      unidade = Metadata.getInstance<Select>('CDFILIAL2');
      unidadeValue = (unidade.value) ? unidade.value : 'T';
    } else {
      unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
      unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    }
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPSERV');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    // const cdfilcentpme = this.appController.getCdCentral();
    const idTipoServ = this.utilitiesPLA.isMerenda() ? '' : 'N';
    
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: idTipoServ,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public controllPratoIni() {
    const pratoFinal = Metadata.getInstance<SelectTree>('PRATO_FINAL');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
  }
  
  public verifyPratIni() {
    const component = Metadata.getInstance<SelectTree>('PRATO_FINAL');
    // const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    if (component && (component.value !== null && component.value !== 'undefined')) {
      const pratoIni = Metadata.getInstance<SelectTree>('PRATO_INICIAL');
      if (this.utilities.returnPrat(pratoIni.value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
        return false;
      }
    }
    
    return true;
  }
  
  public verifyProductIni() {
    if (Metadata.getInstance<SelectTree>('PRODINI')) {
      const productIni = Metadata.getInstance<SelectTree>('PRODINI');
      const productFin = Metadata.getInstance<SelectTree>('PRODFIN');
      
      if (productIni.value > productFin.value) {
        this.utilities.showAlert('Alert', 'pme::prodIniMaior', 'error');
        productFin.setValue(null);
        return false;
      }
    }
    
    return true;
  }
  
  public controllProductIni() {
    const productFin = Metadata.getInstance<SelectTree>('PRODFIN');
    if (productFin.value !== null) productFin.setValue(null);
  }
  
  public setOrdena() {
    const ordena = Metadata.getInstance<Radio>('ORDENA');
    const ordenaAlf = Metadata.getInstance<Select>('NIVEL');
      
    if (ordena.value === '1') {
      ordenaAlf.disabled = false;
      ordenaAlf.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else {
      ordenaAlf.disabled = true;
      ordenaAlf.removeValidation('required');
    }
  }

  public validaStatus() {
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const component = Metadata.getInstance<Button>('pdf');
    
    const validPrat = this.verifyPratIni();
    const validProd = this.verifyProductIni();
    
    if (validPrat && validProd) {
      if (this.utilities.centralIsValid() && form.validate()) {
        const row = form.value;
        this.setParams(row);
        if (!row.CDFILCENTPME && this.project === 'PAE') {
          this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
        } else {
          this.reportController.callGenerateReport(component, row, 'report/emissaoReceitas');
        }
      }
    } else {
      const pratoFin = Metadata.getInstance<SelectTree>('PRATO_FINAL');
      const prodFin = Metadata.getInstance<SelectTree>('PRODFIN');
      pratoFin.setValue(null);
      prodFin.setValue(null);
      this.utilities.disableLoadingButton(component);
    }
  }
  
  public validaPDF() {
    const status = Metadata.getInstance<CheckboxMultiple>('IDTPREQPRODFILI');
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE;
    
    if (report === 'pla05400') {
      if (status.value.length === 0) {
        this.utilities.showAlert('Alert', 'pme::statusPreencher', 'error');
        return;
      }
    }
      
    this.validaStatus();
  }
  
  private setParams(row: IDictionary) {
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    
    const maskProd: string = (Config.masks.productMask as PmeMask).pattern;
    const maskPrat: string = (Config.masks.plateMask as PmeMask).pattern;
    row.CDPICTPROD = this.utilities.formatReportMask(maskProd);
    row.CDPICTPRAT = this.utilities.formatReportMask(maskPrat);

    row.CDFILCENTPME = this.appController.getCdCentral();
    
    const form = Metadata.getInstance<Form>('FORMEMISSAORECEITAS');
    const report = form.value.REPORT.CD_REPORT_CODE;
    
    row.IDTIPORECE = row.IDTIPORECE ? row.IDTIPORECE : 'P';
    row.GRUPFILI = row.GRUPFILI ? row.GRUPFILI : null;
    
    row.TIPOCARD = row.CDTIPOCARD1 ? row.CDTIPOCARD1 : null;
    row.CDFILIALAUX = this.utilities.getCdCentralLogada;
    
    if (report === 'pla01306') {
      row.CDFILIAL = row.CDFILIAL2;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'PLA01300') {
      row.CDFILIAL = this.utilities.getCdCentralLogada;
      row.CDFILCENTPME = this.central;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'pla05000') {
      if (row.IDTIPORECE === 'P') {
        row.CDFILIAL = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
        row.CDFILIALAUX = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
      } else if (row.IDTIPORECE === 'T') {
        row.CDFILIAL = this.central;
      } else {
        row.CDFILIAL = row.CDFILIAL2;
        row.CDFILIALAUX = row.CDFILIAL2;
      }
      
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI2 : 'X';
    } else if (report === 'pla05500') {
      row.CDFILIAL = row.CDFILIAL1;
      row.TIPOCARD = row.CDTIPOCARD2 ? row.CDTIPOCARD2 : null;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'pla013B') {
      if (row.IDTIPORECE === 'P') {
        row.CDFILIAL = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
        row.CDFILIALAUX = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
      } else if (row.IDTIPORECE === 'T') {
        row.CDFILIAL = this.central;
      } else {
        row.CDFILIAL = row.CDFILIAL2;
      }
      
      // row.CDFILIAL = row.CDFILIAL2 ? row.CDFILIAL2 : this.utilities.getCdCentralLogada;
      row.CDSERVFILI = row.CDSERVFILI2 ? row.CDSERVFILI2 : 'T';
    } else {
      row.CDFILIAL = row.CDFILIAL1;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    }
    
    row.PRATOINI = row.PRATO_INICIAL ? row.PRATO_INICIAL : '';
    row.PRATOFIM = row.PRATO_FINAL ? row.PRATO_FINAL : '';
    row.MAXNIVEL = (Config.masks.plateMask as PmeMask).nivelMax;
    
    row.PRODINI = row.PRODINI ? row.PRODINI : '';
    row.PRODFIN = row.PRODFIN ? row.PRODFIN : '';
    
    row.CDFILIPROC = this.utilities.getCdCentralLogada;
    
    row.GRUPSERV = row.GRUPSERV ? row.GRUPSERV : null;
    row.IDTIPOSERV = this.project === 'PAE' ? 'M' : 'N';
    
    row.ORDENA = row.ORDENA ? row.ORDENA : '0';
    
    row.DTINI = row.DTINI ? row.DTINI : null;
    row.DTFIN = row.DTFIN ? row.DTFIN : null;
    
    row.CDNUTRIENTE = row.CDNUTRIENTE ? row.CDNUTRIENTE : null;
    row.PROJECT = this.project;
    row.IDTPREQPRODFILI = row.IDTPREQPRODFILI ? row.IDTPREQPRODFILI : null;
  }
  
  public dataSourceGet({ component }: IEventParam<SelectTree>) {
    const tprece = Metadata.getInstance<Form>('FORMEMISSAORECEITAS').value.IDTIPORECE;
    const unidade = Metadata.getInstance<Select>('CDFILIAL2');
      
    if (component.datasource !== undefined) {
      component.datasource.filter = {
        CDFILIAL: unidade !== null && tprece === 'L' ? unidade : null,
      };
    }
  }
}
